@@ -5,124 +5,29 @@ use warnings;
 
 use Carp qw/croak/;
 
-use JSON::MaybeXS;
-use LWP::UserAgent;
+use Types::Standard qw(Int Num Str);
 
-use OpenAI::API::Resource::Chat;
-use OpenAI::API::Resource::Completion;
-use OpenAI::API::Resource::Edit;
-use OpenAI::API::Resource::Embedding;
-use OpenAI::API::Resource::Model;
-use OpenAI::API::Resource::Moderation;
+use Moo;
+use strictures 2;
+use namespace::clean;
 
-our $VERSION = 0.23;
+with 'OpenAI::API::ResourceDispatcherRole';
+
+our $VERSION = 0.24;
 
 my $DEFAULT_API_BASE = 'https://api.openai.com/v1';
-my $DEFAULT_TIMEOUT  = 60;
-my $DEFAULT_RETRIES  = 3;
-my $DEFAULT_SLEEP    = 1;
 
-sub new {
-    my ( $class, %params ) = @_;
-    my $self = {
-        api_key  => $params{api_key}  // $ENV{OPENAI_API_KEY},
-        api_base => $params{api_base} // $ENV{OPENAI_API_BASE} // $DEFAULT_API_BASE,
-        timeout  => $params{timeout}  // $DEFAULT_TIMEOUT,
-        retry    => $params{retry}    // $DEFAULT_RETRIES,
-        sleep    => $params{sleep}    // $DEFAULT_SLEEP,
-    };
+has api_key  => ( is => 'rw', isa => Str, default => sub { $ENV{OPENAI_API_KEY} }, required => 1 );
+has api_base => ( is => 'rw', isa => Str, default => sub { $ENV{OPENAI_API_BASE} // $DEFAULT_API_BASE }, );
+has timeout  => ( is => 'rw', isa => Num, default => sub { 60 } );
+has retry    => ( is => 'rw', isa => Int, default => sub { 3 } );
+has sleep    => ( is => 'rw', isa => Num, default => sub { 1 } );
 
-    $self->{ua} = LWP::UserAgent->new( timeout => $params{timeout} );
+has user_agent => ( is => 'lazy' );
 
-    croak 'Missing OPENAI_API_KEY' if !defined $self->{api_key};
-    return bless $self, $class;
-}
-
-sub chat {
-    my ( $self, %params ) = @_;
-    my $request = OpenAI::API::Resource::Chat->new( \%params );
-    return $self->_post( $request );
-}
-
-sub completions {
-    my ( $self, %params ) = @_;
-    my $request = OpenAI::API::Resource::Completion->new( \%params );
-    return $self->_post( $request );
-}
-
-sub edits {
-    my ( $self, %params ) = @_;
-    my $request = OpenAI::API::Resource::Edit->new( \%params );
-    return $self->_post( $request );
-}
-
-sub embeddings {
-    my ( $self, %params ) = @_;
-    my $request = OpenAI::API::Resource::Embedding->new( \%params );
-    return $self->_post( $request );
-}
-
-sub moderations {
-    my ( $self, %params ) = @_;
-    my $request = OpenAI::API::Resource::Moderation->new( \%params );
-    return $self->_post( $request );
-}
-
-sub models {
-    my ( $self, %params ) = @_;
-    my $request = OpenAI::API::Resource::Model->new( \%params );
-    return $self->_get($request);
-}
-
-sub _get {
-    my ( $self, $resource ) = @_;
-
-    my $method = $resource->endpoint();
-    my %params = %{$resource};
-
-    my $req = HTTP::Request->new(
-        GET => "$self->{api_base}/$method",
-        [
-            'Content-Type'  => 'application/json',
-            'Authorization' => "Bearer $self->{api_key}",
-        ],
-    );
-
-    return $self->_http_send_request($req);
-}
-
-sub _post {
-    my ( $self, $resource ) = @_;
-
-    my $method = $resource->endpoint();
-    my %params = %{$resource};
-
-    my $req = HTTP::Request->new(
-        POST => "$self->{api_base}/$method",
-        [
-            'Content-Type'  => 'application/json',
-            'Authorization' => "Bearer $self->{api_key}",
-        ],
-        encode_json(\%params),
-    );
-
-    return $self->_http_send_request($req);
-}
-
-sub _http_send_request {
-    my ($self, $req) = @_;
-
-    for my $attempt ( 1 .. $self->{retry} ) {
-        my $res = $self->{ua}->request($req);
-
-        if ( $res->is_success ) {
-            return decode_json( $res->decoded_content );
-        } elsif ( $res->code =~ /^(?:500|503|504|599)$/ && $attempt < $self->{retry} ) {
-            sleep( $self->{sleep} );
-        } else {
-            die "Error: '@{[ $res->status_line ]}'";
-        }
-    }
+sub _build_user_agent {
+    my ($self) = @_;
+    $self->{user_agent} = LWP::UserAgent->new( timeout => $self->timeout );
 }
 
 1;
